==> otrs_1.3.2p01.orig.tar.gz: otrs/Kernel/Language.pm <==
# --
# Kernel/Language.pm - provides multi language support
# Copyright (C) 2001-2004 Martin Edenhofer <martin+code@otrs.org>
# --
# $Id: Language.pm,v 1.33 2004/06/15 09:16:34 martin Exp $
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (GPL). If you
# did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
# --

package Kernel::Language;

use strict;
use Kernel::System::Encode;
use Kernel::System::Time;

use vars qw(@ISA $VERSION);

$VERSION = '$Revision: 1.33 $';
$VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;

=head1 NAME

Kernel::Language - global language interface

=head1 SYNOPSIS

All language functions. 

=head1 PUBLIC INTERFACE

=over 4

=cut

=item new()

create a language object 
 
  use Kernel::Config;
  use Kernel::System::Log;
  use Kernel::Langauge;

  my $ConfigObject = Kernel::Config->new();
  my $LogObject    = Kernel::System::Log->new(
      ConfigObject => $ConfigObject,
  );

  my $LangaugeObject = Kernel::Langauge->new( 
      ConfigObject => $ConfigObject,
      LogObject => $LogObject,
      UserLanguage => 'de',
  );

=cut

# --
sub new {
    my $Type = shift;
    my %Param = @_;

    # allocate new hash for object
    my $Self = {}; 
    bless ($Self, $Type);

    # get common objects 
    foreach (keys %Param) {
        $Self->{$_} = $Param{$_};
    }
    # check needed objects
    foreach (qw(ConfigObject LogObject)) {
        die "Got no $_!" if (!$Self->{$_});
    } 
    # encode object
    $Self->{EncodeObject} = Kernel::System::Encode->new(%Param);
    # time object
    $Self->{TimeObject} = Kernel::System::Time->new(%Param);
    # 0=off; 1=on; 2=get all not translated words; 3=get all requests
    $Self->{Debug} = 0;
    # user language
    $Self->{UserLanguage} = $Param{UserLanguage} || $Self->{ConfigObject}->Get('DefaultLanguage') || 'en';
#    $Self->{UserLanguage} = 'english';
    # Debug 
    if ($Self->{Debug} > 0) {
        $Self->{LogObject}->Log(
          Priority => 'Debug',
          Message => "UserLanguage = $Self->{UserLanguage}",
        );
    }
    # load text catalog ...
    if (eval "require Kernel::Language::$Self->{UserLanguage}") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage} load ... done."
            );
        }
    }
    # if there is no translation
    else {
        $Self->{LogObject}->Log(
          Priority => 'Error',
          Message => "Sorry, can't locate or load Kernel::Language::$Self->{UserLanguage} ".
              "translation! Check the Kernel/Language/$Self->{UserLanguage}.pm (perl -cw)!",
        );
    }
    # load action text catalog ...
    if ($Param{Action} && eval "require Kernel::Language::$Self->{UserLanguage}_$Param{Action}") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}_$Param{Action}");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage}_$Param{Action} load ... done."
            );
        }
    }
    # load custom text catalog ...
    if (eval "require Kernel::Language::$Self->{UserLanguage}_Custom") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}_Custom");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage}_Custom load ... done."
            );
        }
    }
    # if no return charset is given, use recommended return charset
    if (!$Self->{ReturnCharset}) {
        $Self->{ReturnCharset} = $Self->GetRecommendedCharset();
    }
    # get source file charset
    # what charset shoud I use (take it from translation file)!
    if ($Self->{Charset}) {
        my @Chatsets = @{$Self->{Charset}};
        $Self->{TranslationCharset} = $Chatsets[$#Chatsets];
    }
    # remember if a translation file should be written
    $Self->{WriteNewTranslationFile} = $Self->{ConfigObject}->Get('WriteNewTranslationFile');
    return $Self;
}
# --

=item Get()
    
Translate a words.

  my $Text = $LanguageObject->Get('Hello');

=cut

sub Get {
    my $Self = shift;
    my $What = shift;
    my $File = shift || '';
    my @Dyn = ();
    # --
    # check
    # --
    if (! defined $What) {
        return;
    }
    # --
    # check dyn spaces
    # --
    if ($What && $What =~ /^(.+?)", "(.+?|)$/) {
        $What = $1;
        @Dyn = split(/", "/, $2);
    }
    # --
    # check wanted param and returns the 
    # lookup or the english data
    # --
    if (exists $Self->{Translation}->{$What} && $Self->{Translation}->{$What} ne '') {
        # Debug
        if ($Self->{Debug} > 3) {
            $Self->{LogObject}->Log(
              Priority => 'Debug',
              Message => "->Get('$What') = ('$Self->{Translation}->{$What}').",
            );
        }
        # charset convert from source translation into shown charset
        if (!$Self->{Translation}->{Convert}->{$What}) {
            # remember that charset convert is already done
            $Self->{Translation}->{Convert}->{$What} = 1;
            # convert it
            $Self->{Translation}->{$What} = $Self->CharsetConvert(
                Text => $Self->{Translation}->{$What},
                From => $Self->{TranslationCharset},
            );
        }
        # remember translated words
        if ($Self->{WriteNewTranslationFile}) {
            if ($Self->{UsedWords}->{$File}) {
               $Self->{UsedWords}->{$File} = {$What => $Self->{Translation}->{$What}, %{$Self->{UsedWords}->{$File}}};
            }
            else {
               $Self->{UsedWords}->{$File} = {$What => $Self->{Translation}->{$What}};
            }
        }
        my $Text = $Self->{Translation}->{$What};
        foreach (0..3) {
            if (defined $Dyn[$_]) {
                if ($Dyn[$_] =~ /Time\((.*)\)/) {
                    $Dyn[$_] = $Self->Time(
                        Action => 'GET', 
                        Format => $1,
                    );
                    $Text =~ s/\%(s|d)/$Dyn[$_]/;
                }
                else {
                    $Text =~ s/\%(s|d)/$Dyn[$_]/;
                }
            }
        }
        return $Text; 
    }
    else {
        # warn if the value is not def
        if ($Self->{Debug} > 1) {
          $Self->{LogObject}->Log(
            Priority => 'debug',
            Message => "->Get('$What') Is not translated!!!",
          );
        }
        # remember not translated words
        if ($Self->{WriteNewTranslationFile}) {
            if ($Self->{UsedWords}->{$File}) {
                $Self->{UsedWords}->{$File} = {$What => '', %{$Self->{UsedWords}->{$File}}};
            }
            else {
                $Self->{UsedWords}->{$File} = {$What => ''};
            }
        }
        foreach (0..3) {
            if (defined $Dyn[$_]) {
                if ($Dyn[$_] =~ /Time\((.*)\)/) {
                    $Dyn[$_] = $Self->Time(
                        Action => 'GET', 
                        Format => $1,
                    );
                    $What =~ s/\%(s|d)/$Dyn[$_]/; 
                }
                else {
                    $What =~ s/\%(s|d)/$Dyn[$_]/; 
                }
            }
        }
        return $What;
    }
}
# --

=item FormatTimeString()

Get date format in used language formate (based on translation file).
    
  my $Date = $LanguageObject->FormatTimeString('2005-12-12 12:12:12', 'DateFormat');

=cut

sub FormatTimeString {
    my $Self = shift;
    my $String = shift || return;
    my $Config = shift || 'DateFormat';
    my $ReturnString = $Self->{$Config} || "$Config needs to be translated!";
    if ($String =~ /(\d\d\d\d)-(\d\d)-(\d\d)\s(\d\d:\d\d:\d\d)/) {
        my ($Y,$M,$D, $T) = ($1, $2, $3, $4);
        $ReturnString =~ s/\%T/$T/g;
        $ReturnString =~ s/\%D/$D/g;
        $ReturnString =~ s/\%M/$M/g;
        $ReturnString =~ s/\%Y/$Y/g;
        return $ReturnString;
    }
    else {
        $Self->{LogObject}->Log(
            Priority => 'notice', 
            Message => "No FormatTimeString() translation found for '$String' string!",
        );
        return $String;
    }
}
# --

=item GetRecommendedCharset()

Returns the recommended charset for frontend (based on translation
file or from DefaultCharset (from Kernel/Config.pm) is utf-8).
    
  my $Charset = $LanguageObject->GetRecommendedCharset().

=cut

sub GetRecommendedCharset {
    my $Self = shift;
    # should I use default frontend charset (e. g. utf-8)?
    if ($Self->{EncodeObject}->EncodeFrontendUsed()) {
        return $Self->{EncodeObject}->EncodeFrontendUsed();
    }
    # if not, what charset shoud I use (take it from translation file)?
    if ($Self->{Charset}) {
        my @Chatsets = @{$Self->{Charset}};
        return $Chatsets[$#Chatsets];
    }
    else {
        return $Self->{ConfigObject}->Get('DefaultCharset') || 'iso-8859-1';
    }
}
# --

=item GetPossibleCharsets()

Returns an array of possible charsets (based on translation file).
    
  my @Charsets = $LanguageObject->GetPossibleCharsets().

=cut

sub GetPossibleCharsets {
    my $Self = shift;
    if ($Self->{Charset}) {
        return @{$Self->{Charset}};
    }
    else {
        return;
    }
}
# --

=item Time()

Returns a time string in language formate (based on translation file).
   
    $Time = $LanguageObject->Time(
        Action => 'GET',
        Format => 'DateFormat',
    );

    $TimeLong = $LanguageObject->Time(
        Action => 'GET',
        Format => 'DateFormatLong',
    );
 
    $TimeLong = $LanguageObject->Time(
        Action => 'RETURN',
        Format => 'DateFormatLong',
        Year => 1977,
        Month => 10,
        Day => 27,
        Hour => 20,
        Minute => 10,
    );
 
=cut

sub Time {
    my $Self = shift;
    my %Param = @_;
    # check needed stuff
    foreach (qw(Action Format)) {
      if (!$Param{$_}) {
        $Self->{LogObject}->Log(Priority => 'error', Message => "Need $_!");
        return;
      }
    }
    my $ReturnString = $Self->{$Param{Format}} || 'Need to be translated!';
    my ($s,$m,$h, $D,$M,$Y, $wd,$yd,$dst);
    # set or get time
    if ($Param{Action} =~ /^GET$/i) {
        my @DAYS = qw/Sun Mon Tue Wed Thu Fri Sat/;
        my @MONS = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
        ($s,$m,$h, $D,$M,$Y, $wd,$yd,$dst) = $Self->{TimeObject}->SystemTime2Date(
            SystemTime => $Self->{TimeObject}->SystemTime(),
        );
    }
    elsif ($Param{Action} =~ /^RETURN$/i) {
        $m = $Param{Minute} || 0;
        $h = $Param{Hour} || 0;
        $D = $Param{Day} || 0;
        $M = $Param{Month} || 0;
        $Y = $Param{Year} || 0;
    }
    # do replace
    if ($Param{Action} =~ /^(GET|RETURN)$/i) {
        my @DAYS = qw/Sun Mon Tue Wed Thu Fri Sat/;
        my @MONS = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
        my $Time = '';
        if ($Param{Mode} && $Param{Mode} =~ /^NotNumeric$/i) {
            if (!$s) {
                $Time = "$h:$m";
            }
            else {
                $Time = "$h:$m:$s";
            }
        }
        else {
            $Time = sprintf("%02d:%02d:%02d", $h,$m,$s);
            $D = sprintf("%02d", $D);
            $M = sprintf("%02d", $M);
        }
        $ReturnString =~ s/\%T/$Time/g;
        $ReturnString =~ s/\%D/$D/g;
        $ReturnString =~ s/\%M/$M/g;
        $ReturnString =~ s/\%Y/$Y/g;
        $ReturnString =~ s/\%Y/$Y/g;
        $ReturnString =~ s{(\%A)}{$Self->Get($DAYS[$wd]);}egx;
        $ReturnString =~ s{(\%B)}{$Self->Get($MONS[$M-1]);}egx;
        return $ReturnString;
    } 
    # return
    return $ReturnString;
}
# --

=item CharsetConvert()

Converts charset from a source string (if no To is given, the the 
GetRecommendedCharset() will be used).
    
  my $Text = $LanguageObject->CharsetConvert(
      Text => $String,
      From => 'iso-8859-15',
      To => 'utf-8',
  );

=cut

sub CharsetConvert {
    my $Self = shift;
    my %Param = @_;
    my $Text = defined $Param{Text} ? $Param{Text} : return;
    my $From = $Param{From} || return $Text;
    my $To = $Param{To} || $Self->{ReturnCharset} || return $Text;
    $From =~ s/'|"//g;
    # encode 
    return $Self->{EncodeObject}->Convert(
        From => $From,
        To => $To,
        Text => $Text,
    );
}
# --
sub DESTROY {
    my $Self = shift;
    if (!$Self->{WriteNewTranslationFile}) {
        return 1;
    }
    if ($Self->{UsedWords}) {
        my %UniqWords = ();
        my $Data = '';
        my %Screens = %{$Self->{UsedWords}};
        $Data .= "    # possible charsets\n".
                 "    \$Self->{Charset} = [";
        if ($Self->{Charset}) {
            foreach (@{$Self->{Charset}}) {
                $Data .= "'$_', ";
            }
        }
        $Data .= "];\n".
                 "    # date formats (\%A=WeekDay;\%B=LongMonth;\%T=Time;\%D=Day;\%M=Month;\%Y=Jear;)\n".
                 "    \$Self->{DateFormat} = '$Self->{DateFormat}';\n".
                 "    \$Self->{DateFormatLong} = '$Self->{DateFormatLong}';\n".
                 "    \$Self->{DateInputFormat} = '$Self->{DateInputFormat}';\n".
                 "    \$Self->{DateInputFormatLong} = '$Self->{DateInputFormatLong}';\n\n".
                 "    \%Hash = (";
        foreach my $Screen (sort keys %Screens) {
            my %Words = %{$Screens{$Screen}};
            if ($Screen) {
                $Data .= "\n    # Template: $Screen\n";
                foreach my $Key (sort {uc($a) cmp uc($b)} keys %Words) {
                    if (!$UniqWords{$Key} && $Key) {
                        $UniqWords{$Key} = 1;
                        my $QuoteKey = $Key;
                        $QuoteKey =~ s/'/\\'/g;
                        if (defined $Words{$Key}) {
                            $Words{$Key} =~ s/'/\\'/g;   
                        }
                        else {
                            $Words{$Key} = '';
                        }
                        $Data .= "      '$QuoteKey' => '$Words{$Key}',\n";
                    }
                }
            }
        }
        $Data .= "\n    # Misc\n";
        foreach my $Key (sort keys %{$Self->{Translation}}) {
            if (!$UniqWords{$Key} && $Key && $Self->{Translation}->{$Key} !~ /HASH\(/) {
                $UniqWords{$Key} = 1;
                my $QuoteKey = $Key;
                $QuoteKey =~ s/'/\\'/g;
                if (defined $Self->{Translation}->{$Key}) {
                    $Self->{Translation}->{$Key} =~ s/'/\\'/g;
                }
                else {
                    $Self->{Translation}->{$Key} = '';
                }
                $Data .= "      '$QuoteKey' => '$Self->{Translation}->{$Key}',\n";
            }
        }
        $Data .= "    );\n";
        return $Data;
    }
}
# --
1;

=head1 TERMS AND CONDITIONS

This software is part of the OTRS project (http://otrs.org/).  

This software comes with ABSOLUTELY NO WARRANTY. For details, see
the enclosed file COPYING for license information (GPL). If you
did not receive this file, see http://www.gnu.org/licenses/gpl.txt.

=cut

=head1 VERSION

$Revision: 1.33 $ $Date: 2004/06/15 09:16:34 $

=cut

==> otrs2_2.0.4p01.orig.tar.gz: otrs/Kernel/Language.pm <==
# --
# Kernel/Language.pm - provides multi language support
# Copyright (C) 2001-2005 Martin Edenhofer <martin+code@otrs.org>
# --
# $Id: Language.pm,v 1.37 2005/06/17 16:11:09 martin Exp $
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (GPL). If you
# did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
# --

package Kernel::Language;

use strict;
use Kernel::System::Encode;
use Kernel::System::Time;

use vars qw(@ISA $VERSION);

$VERSION = '$Revision: 1.37 $';
$VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;

=head1 NAME

Kernel::Language - global language interface

=head1 SYNOPSIS

All language functions.

=head1 PUBLIC INTERFACE

=over 4

=cut

=item new()

create a language object

  use Kernel::Config;
  use Kernel::System::Log;
  use Kernel::Langauge;

  my $ConfigObject = Kernel::Config->new();
  my $LogObject    = Kernel::System::Log->new(
      ConfigObject => $ConfigObject,
  );

  my $LangaugeObject = Kernel::Langauge->new(
      ConfigObject => $ConfigObject,
      LogObject => $LogObject,
      UserLanguage => 'de',
  );

=cut

sub new {
    my $Type = shift;
    my %Param = @_;

    # allocate new hash for object
    my $Self = {};
    bless ($Self, $Type);

    # get common objects
    foreach (keys %Param) {
        $Self->{$_} = $Param{$_};
    }
    # check needed objects
    foreach (qw(ConfigObject LogObject)) {
        die "Got no $_!" if (!$Self->{$_});
    }
    # encode object
    $Self->{EncodeObject} = Kernel::System::Encode->new(%Param);
    # time object
    $Self->{TimeObject} = Kernel::System::Time->new(%Param);
    # 0=off; 1=on; 2=get all not translated words; 3=get all requests
    $Self->{Debug} = 0;
    # user language
    $Self->{UserLanguage} = $Param{UserLanguage} || $Self->{ConfigObject}->Get('DefaultLanguage') || 'en';
#    $Self->{UserLanguage} = 'english';
    # Debug
    if ($Self->{Debug} > 0) {
        $Self->{LogObject}->Log(
          Priority => 'Debug',
          Message => "UserLanguage = $Self->{UserLanguage}",
        );
    }
    # load text catalog ...
    if (eval "require Kernel::Language::$Self->{UserLanguage}") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage} load ... done."
            );
        }
    }
    # if there is no translation
    else {
        $Self->{LogObject}->Log(
          Priority => 'Error',
          Message => "Sorry, can't locate or load Kernel::Language::$Self->{UserLanguage} ".
              "translation! Check the Kernel/Language/$Self->{UserLanguage}.pm (perl -cw)!",
        );
    }
    # load action text catalog ...
    if (!$Param{TranslationFile} && $Param{Action} && eval "require Kernel::Language::$Self->{UserLanguage}_$Param{Action}") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}_$Param{Action}");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage}_$Param{Action} load ... done."
            );
        }
    }
    # load custom text catalog ...
    if (!$Param{TranslationFile} && eval "require Kernel::Language::$Self->{UserLanguage}_Custom") {
       @ISA = ("Kernel::Language::$Self->{UserLanguage}_Custom");
       $Self->Data();
       if ($Self->{Debug} > 0) {
            $Self->{LogObject}->Log(
                Priority => 'Debug',
                Message => "Kernel::Language::$Self->{UserLanguage}_Custom load ... done."
            );
        }
    }
    # if no return charset is given, use recommended return charset
    if (!$Self->{ReturnCharset}) {
        $Self->{ReturnCharset} = $Self->GetRecommendedCharset();
    }
    # get source file charset
    # what charset shoud I use (take it from translation file)!
    if ($Self->{Charset}) {
        my @Chatsets = @{$Self->{Charset}};
        $Self->{TranslationCharset} = $Chatsets[$#Chatsets];
    }
    return $Self;
}

=item Get()

Translate a words.

  my $Text = $LanguageObject->Get('Hello');

=cut

sub Get {
    my $Self = shift;
    my $What = shift;
    my $File = shift || '';
    my @Dyn = ();
    # --
    # check
    # --
    if (! defined $What) {
        return;
    }
    # --
    # check dyn spaces
    # --
    if ($What && $What =~ /^(.+?)", "(.+?|)$/) {
        $What = $1;
        @Dyn = split(/", "/, $2);
    }
    # --
    # check wanted param and returns the
    # lookup or the english data
    # --
    if (exists $Self->{Translation}->{$What} && $Self->{Translation}->{$What} ne '') {
        # Debug
        if ($Self->{Debug} > 3) {
            $Self->{LogObject}->Log(
              Priority => 'Debug',
              Message => "->Get('$What') = ('$Self->{Translation}->{$What}').",
            );
        }
        # charset convert from source translation into shown charset
        if (!$Self->{TranslationConvert}->{$What}) {
            # remember that charset convert is already done
            $Self->{TranslationConvert}->{$What} = 1;
            # convert it
            $Self->{Translation}->{$What} = $Self->CharsetConvert(
                Text => $Self->{Translation}->{$What},
                From => $Self->{TranslationCharset},
            );
        }
        my $Text = $Self->{Translation}->{$What};
        foreach (0..3) {
            if (defined $Dyn[$_]) {
                if ($Dyn[$_] =~ /Time\((.*)\)/) {
                    $Dyn[$_] = $Self->Time(
                        Action => 'GET',
                        Format => $1,
                    );
                    $Text =~ s/\%(s|d)/$Dyn[$_]/;
                }
                else {
                    $Text =~ s/\%(s|d)/$Dyn[$_]/;
                }
            }
        }
        return $Text;
    }
    else {
        # warn if the value is not def
        if ($Self->{Debug} > 1) {
          $Self->{LogObject}->Log(
            Priority => 'debug',
            Message => "->Get('$What') Is not translated!!!",
          );
        }
        foreach (0..3) {
            if (defined $Dyn[$_]) {
                if ($Dyn[$_] =~ /Time\((.*)\)/) {
                    $Dyn[$_] = $Self->Time(
                        Action => 'GET',
                        Format => $1,
                    );
                    $What =~ s/\%(s|d)/$Dyn[$_]/;
                }
                else {
                    $What =~ s/\%(s|d)/$Dyn[$_]/;
                }
            }
        }
        return $What;
    }
}

=item FormatTimeString()

Get date format in used language formate (based on translation file).

  my $Date = $LanguageObject->FormatTimeString('2005-12-12 12:12:12', 'DateFormat');

=cut

sub FormatTimeString {
    my $Self = shift;
    my $String = shift || return;
    my $Config = shift || 'DateFormat';
    my $Short  = shift || 0;
    my $ReturnString = $Self->{$Config} || "$Config needs to be translated!";
    if ($String =~ /(\d\d\d\d)-(\d\d)-(\d\d)\s(\d\d:\d\d:\d\d)/) {
        my ($Y,$M,$D, $T) = ($1, $2, $3, $4);
        if ($Short) {
            $T =~ s/(\d\d:\d\d):\d\d/$1/g;
        }
        $ReturnString =~ s/\%T/$T/g;
        $ReturnString =~ s/\%D/$D/g;
        $ReturnString =~ s/\%M/$M/g;
        $ReturnString =~ s/\%Y/$Y/g;
        return $ReturnString;
    }
    elsif ($String =~ /^(\d\d:\d\d:\d\d)$/) {
        return $String;
    }
    else {
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message => "No FormatTimeString() translation found for '$String' string!",
        );
        return $String;
    }
}

=item GetRecommendedCharset()

Returns the recommended charset for frontend (based on translation
file or from DefaultCharset (from Kernel/Config.pm) is utf-8).

  my $Charset = $LanguageObject->GetRecommendedCharset().

=cut

sub GetRecommendedCharset {
    my $Self = shift;
    # should I use default frontend charset (e. g. utf-8)?
    if ($Self->{EncodeObject}->EncodeFrontendUsed()) {
        return $Self->{EncodeObject}->EncodeFrontendUsed();
    }
    # if not, what charset shoud I use (take it from translation file)?
    if ($Self->{Charset}) {
        my @Chatsets = @{$Self->{Charset}};
        return $Chatsets[$#Chatsets];
    }
    else {
        return $Self->{ConfigObject}->Get('DefaultCharset') || 'iso-8859-1';
    }
}

=item GetPossibleCharsets()

Returns an array of possible charsets (based on translation file).

  my @Charsets = $LanguageObject->GetPossibleCharsets().

=cut

sub GetPossibleCharsets {
    my $Self = shift;
    if ($Self->{Charset}) {
        return @{$Self->{Charset}};
    }
    else {
        return;
    }
}

=item Time()

Returns a time string in language formate (based on translation file).

    $Time = $LanguageObject->Time(
        Action => 'GET',
        Format => 'DateFormat',
    );

    $TimeLong = $LanguageObject->Time(
        Action => 'GET',
        Format => 'DateFormatLong',
    );

    $TimeLong = $LanguageObject->Time(
        Action => 'RETURN',
        Format => 'DateFormatLong',
        Year => 1977,
        Month => 10,
        Day => 27,
        Hour => 20,
        Minute => 10,
    );

=cut

sub Time {
    my $Self = shift;
    my %Param = @_;
    # check needed stuff
    foreach (qw(Action Format)) {
      if (!$Param{$_}) {
        $Self->{LogObject}->Log(Priority => 'error', Message => "Need $_!");
        return;
      }
    }
    my $ReturnString = $Self->{$Param{Format}} || 'Need to be translated!';
    my ($s,$m,$h, $D,$M,$Y, $wd,$yd,$dst);
    # set or get time
    if ($Param{Action} =~ /^GET$/i) {
        my @DAYS = qw/Sun Mon Tue Wed Thu Fri Sat/;
        my @MONS = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
        ($s,$m,$h, $D,$M,$Y, $wd,$yd,$dst) = $Self->{TimeObject}->SystemTime2Date(
            SystemTime => $Self->{TimeObject}->SystemTime(),
        );
    }
    elsif ($Param{Action} =~ /^RETURN$/i) {
        $m = $Param{Minute} || 0;
        $h = $Param{Hour} || 0;
        $D = $Param{Day} || 0;
        $M = $Param{Month} || 0;
        $Y = $Param{Year} || 0;
    }
    # do replace
    if ($Param{Action} =~ /^(GET|RETURN)$/i) {
        my @DAYS = qw/Sun Mon Tue Wed Thu Fri Sat/;
        my @MONS = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
        my $Time = '';
        if ($Param{Mode} && $Param{Mode} =~ /^NotNumeric$/i) {
            if (!$s) {
                $Time = "$h:$m";
            }
            else {
                $Time = "$h:$m:$s";
            }
        }
        else {
            $Time = sprintf("%02d:%02d:%02d", $h,$m,$s);
            $D = sprintf("%02d", $D);
            $M = sprintf("%02d", $M);
        }
        $ReturnString =~ s/\%T/$Time/g;
        $ReturnString =~ s/\%D/$D/g;
        $ReturnString =~ s/\%M/$M/g;
        $ReturnString =~ s/\%Y/$Y/g;
        $ReturnString =~ s/\%Y/$Y/g;
        $ReturnString =~ s{(\%A)}{$Self->Get($DAYS[$wd]);}egx;
        $ReturnString =~ s{(\%B)}{$Self->Get($MONS[$M-1]);}egx;
        return $ReturnString;
    }
    # return
    return $ReturnString;
}

=item CharsetConvert()

Converts charset from a source string (if no To is given, the the
GetRecommendedCharset() will be used).

  my $Text = $LanguageObject->CharsetConvert(
      Text => $String,
      From => 'iso-8859-15',
      To => 'utf-8',
  );

=cut

sub CharsetConvert {
    my $Self = shift;
    my %Param = @_;
    my $Text = defined $Param{Text} ? $Param{Text} : return;
    my $From = $Param{From} || return $Text;
    my $To = $Param{To} || $Self->{ReturnCharset} || return $Text;
    $From =~ s/'|"//g;
    # encode
    return $Self->{EncodeObject}->Convert(
        From => $From,
        To => $To,
        Text => $Text,
    );
}

1;

=head1 TERMS AND CONDITIONS

This software is part of the OTRS project (http://otrs.org/).

This software comes with ABSOLUTELY NO WARRANTY. For details, see
the enclosed file COPYING for license information (GPL). If you
did not receive this file, see http://www.gnu.org/licenses/gpl.txt.

=cut

=head1 VERSION

$Revision: 1.37 $ $Date: 2005/06/17 16:11:09 $

=cut
